@@ -32,7 +32,7 @@ class CSRF(object):
         if rq.method == 'POST':
             if not getattr(func, '_pysi_csrf_exempt', None):
                 rq_token = (rq.POST.get(cfg.CSRF_TOKEN_POST_NAME)
-                        or rq.environ.get('HTTP_X_CSRFTOKEN'))
+                        or rq.environ.get('HTTP_X_CSRF_TOKEN'))
                 if not token or token != rq_token:
                     logging.debug('Bad CSRF token. Expected: %s Got: %s' % (
                         token, rq_token))
