@@ -7,7 +7,9 @@ from datetime import timedelta
 from hashlib import sha1
 from StringIO import StringIO
 from tempfile import TemporaryFile
-from Cookie import SimpleCookie
+from Cookie import SimpleCookie, CookieError
+
+import simplejson as json
 
 from util import cached_property, make_traceback, obj_from_str, list_obj_from_str
 from config import cfg
@@ -45,6 +47,31 @@ class Request(object):
             v[-1].decode(self.charset, self.encoding_errors))
                 for k, v in cgi.parse_qs(self.environ['QUERY_STRING']).iteritems())
 
+    @cached_property
+    def data(self):
+        # создаём файловый объект в памяти или на диске в зависимости от размера
+        try:
+            maxread = int(self.environ.get('CONTENT_LENGTH', 0))
+        except ValueError:
+            maxread = 0
+        if self.max_post_size and maxread > self.max_post_size:
+            abort(413)
+        stream = self.environ['wsgi.input']
+        if cfg.POST_MAX_MEMFILE is None or maxread < cfg.POST_MAX_MEMFILE:
+            body = StringIO()
+        else:
+            body = TemporaryFile(mode='w+b')
+        while maxread > 0:
+            if cfg.POST_MAX_MEMFILE:
+                part = stream.read(min(maxread, cfg.POST_MAX_MEMFILE))
+            else:
+                part = stream.read()
+            if not part:  # TODO: Wrong content_length. Error? Do nothing?
+                break
+            body.write(part)
+            maxread -= len(part)
+        return body
+
     @cached_property
     def POST(self):
         '''
@@ -54,35 +81,14 @@ class Request(object):
             rq.POST['datafile'].file.read()  # содержимое файла
             rq.POST['datafile'].filename  # имя файла
         '''
+        self.data.seek(0)
         post = MultiDict()
         if self.method == 'POST':
-            # создаём файловый объект в памяти или на диске в зависимости от размера
-            try:
-                maxread = int(self.environ.get('CONTENT_LENGTH', 0))
-            except ValueError:
-                maxread = 0
-            if self.max_post_size and maxread > self.max_post_size:
-                abort(413)
-            stream = self.environ['wsgi.input']
-            if cfg.POST_MAX_MEMFILE is None or maxread < cfg.POST_MAX_MEMFILE:
-                body = StringIO()
-            else:
-                body = TemporaryFile(mode='w+b')
-            while maxread > 0:
-                if cfg.POST_MAX_MEMFILE:
-                    part = stream.read(min(maxread, cfg.POST_MAX_MEMFILE))
-                else:
-                    part = stream.read()
-                if not part:  # TODO: Wrong content_length. Error? Do nothing?
-                    break
-                body.write(part)
-                maxread -= len(part)
-            body.seek(0)
             # обрабатываем этот объект
             safe_env = {'QUERY_STRING' : ''}  # Build a safe environment for cgi
             for key in ('REQUEST_METHOD', 'CONTENT_TYPE', 'CONTENT_LENGTH'):
                 if key in self.environ: safe_env[key] = self.environ[key]
-            data = cgi.FieldStorage(fp=body, environ=safe_env, keep_blank_values=True)
+            data = cgi.FieldStorage(fp=self.data, environ=safe_env, keep_blank_values=True)
             for item in data.list or []:
                 if item.filename:
                     post[item.name] = item  # файл
@@ -98,6 +104,12 @@ class Request(object):
                 ret[k] = v.decode(self.charset, self.encoding_errors)
         return ret
 
+    @cached_property
+    def json(self):
+        self.data.seek(0)
+        if self.environ.get('CONTENT_TYPE', '').startswith('application/json'):
+            return json.loads(self.data.read())
+
     @cached_property
     def files(self):
         ret = MultiDict()
@@ -108,7 +120,10 @@ class Request(object):
 
     @cached_property
     def COOKIES(self):
-        return SimpleCookie(self.environ.get('HTTP_COOKIE', ''))
+        try:
+            return SimpleCookie(self.environ.get('HTTP_COOKIE', ''))
+        except CookieError:
+            return SimpleCookie('')
 
     @cached_property
     def scheme(self):
